@@ -40,13 +40,22 @@ using Banshee.ServiceStack;
 
 namespace Banshee.GStreamer
 {
-    internal delegate void GstPlaybackEosCallback (IntPtr engine);
-    internal delegate void GstPlaybackErrorCallback (IntPtr engine, uint domain, int code, IntPtr error, IntPtr debug);
-    internal delegate void GstPlaybackStateChangedCallback (IntPtr engine, int old_state, int new_state, int pending_state);
-    internal delegate void GstPlaybackIterateCallback (IntPtr engine);
-    internal delegate void GstPlaybackBufferingCallback (IntPtr engine, int buffering_progress);
+    internal enum GstState
+    {
+        VoidPending = 0,
+        Null = 1,
+        Ready = 2,
+        Paused = 3,
+        Playing = 4
+    }
 
-    internal delegate void GstTaggerTagFoundCallback (string tagName, ref GLib.Value value, IntPtr userData);        
+    internal delegate void BansheePlayerEosCallback (IntPtr player);
+    internal delegate void BansheePlayerErrorCallback (IntPtr player, uint domain, int code, IntPtr error, IntPtr debug);
+    internal delegate void BansheePlayerStateChangedCallback (IntPtr player, GstState old_state, GstState new_state, GstState pending_state);
+    internal delegate void BansheePlayerIterateCallback (IntPtr player);
+    internal delegate void BansheePlayerBufferingCallback (IntPtr player, int buffering_progress);
+
+    internal delegate void GstTaggerTagFoundCallback (IntPtr player, string tagName, ref GLib.Value value);
     
     public class PlayerEngine : Banshee.MediaEngine.PlayerEngine, Banshee.MediaEngine.IEqualizer
     {
@@ -57,15 +66,16 @@ namespace Banshee.GStreamer
     
         private HandleRef handle;
         
-        private GstPlaybackEosCallback eos_callback;
-        private GstPlaybackErrorCallback error_callback;
-        private GstPlaybackStateChangedCallback state_changed_callback;
-        private GstPlaybackIterateCallback iterate_callback;
-        private GstPlaybackBufferingCallback buffering_callback;
+        private BansheePlayerEosCallback eos_callback;
+        private BansheePlayerErrorCallback error_callback;
+        private BansheePlayerStateChangedCallback state_changed_callback;
+        private BansheePlayerIterateCallback iterate_callback;
+        private BansheePlayerBufferingCallback buffering_callback;
         private GstTaggerTagFoundCallback tag_found_callback;
         
         private bool buffering_finished;
         private short pending_volume = -1;
+        private bool xid_is_set = false;
         
         public PlayerEngine ()
         {
@@ -86,7 +96,7 @@ namespace Banshee.GStreamer
         
         private void Initialize ()
         {
-            IntPtr ptr = gst_playback_new ();
+            IntPtr ptr = bp_new ();
             
             if (ptr == IntPtr.Zero) {
                 throw new ApplicationException (Catalog.GetString ("Could not initialize GStreamer library"));
@@ -94,23 +104,24 @@ namespace Banshee.GStreamer
             
             handle = new HandleRef (this, ptr);
             
-            gst_playback_get_error_quarks (out GST_CORE_ERROR, out GST_LIBRARY_ERROR, 
+            bp_get_error_quarks (out GST_CORE_ERROR, out GST_LIBRARY_ERROR, 
                 out GST_RESOURCE_ERROR, out GST_STREAM_ERROR);
             
-            eos_callback = new GstPlaybackEosCallback (OnEos);
-            error_callback = new GstPlaybackErrorCallback (OnError);
-            iterate_callback = new GstPlaybackIterateCallback (OnIterate);
-            buffering_callback = new GstPlaybackBufferingCallback (OnBuffering);
+            eos_callback = new BansheePlayerEosCallback (OnEos);
+            error_callback = new BansheePlayerErrorCallback (OnError);
+            state_changed_callback = new BansheePlayerStateChangedCallback (OnStateChange);
+            iterate_callback = new BansheePlayerIterateCallback (OnIterate);
+            buffering_callback = new BansheePlayerBufferingCallback (OnBuffering);
             tag_found_callback = new GstTaggerTagFoundCallback (OnTagFound);
             
-            gst_playback_set_eos_callback (handle, eos_callback);
-            gst_playback_set_iterate_callback (handle, iterate_callback);
-            gst_playback_set_error_callback (handle, error_callback);
-            gst_playback_set_state_changed_callback (handle, state_changed_callback);
-            gst_playback_set_buffering_callback (handle, buffering_callback);
-            gst_playback_set_tag_found_callback (handle, tag_found_callback);
+            bp_set_eos_callback (handle, eos_callback);
+            bp_set_iterate_callback (handle, iterate_callback);
+            bp_set_error_callback (handle, error_callback);
+            bp_set_state_changed_callback (handle, state_changed_callback);
+            bp_set_buffering_callback (handle, buffering_callback);
+            bp_set_tag_found_callback (handle, tag_found_callback);
             
-            OnStateChanged (PlayerEngineState.Ready);
+            OnStateChanged (PlayerState.Ready);
             
             if (pending_volume >= 0) {
                 Volume = (ushort)pending_volume;
@@ -120,13 +131,13 @@ namespace Banshee.GStreamer
         public override void Dispose ()
         {
             base.Dispose ();
-            gst_playback_free (handle);
+            bp_destroy (handle);
         }
         
-        public override void Close ()
+        public override void Close (bool fullShutdown)
         {
-            gst_playback_stop (handle);
-            base.Close ();
+            bp_stop (handle, fullShutdown);
+            base.Close (fullShutdown);
         }
         
         protected override void OpenUri (SafeUri uri)
@@ -134,54 +145,81 @@ namespace Banshee.GStreamer
             // The GStreamer engine can use the XID of the main window if it ever
             // needs to bring up the plugin installer so it can be transient to
             // the main window.
-
-            IPropertyStoreExpose service = ServiceManager.Get<IService> ("GtkElementsService") as IPropertyStoreExpose;
-            if (service != null) {
-                gst_playback_set_gdk_window (handle, service.PropertyStore.Get<IntPtr> ("PrimaryWindow.RawHandle"));
+            if (!xid_is_set) {
+                IPropertyStoreExpose service = ServiceManager.Get<IService> ("GtkElementsService") as IPropertyStoreExpose;
+                if (service != null) {
+                    bp_set_application_gdk_window (handle, service.PropertyStore.Get<IntPtr> ("PrimaryWindow.RawHandle"));
+                }
+                xid_is_set = true;
             }
                 
             IntPtr uri_ptr = GLib.Marshaller.StringToPtrGStrdup (uri.AbsoluteUri);
-            gst_playback_open (handle, uri_ptr);
-            GLib.Marshaller.Free (uri_ptr);
+            try {
+                if (!bp_open (handle, uri_ptr)) {
+                    throw new ApplicationException ("Could not open resource");
+                }
+            } finally {
+                GLib.Marshaller.Free (uri_ptr);
+            }
         }
         
         public override void Play ()
         {
-            gst_playback_play (handle);
-            OnStateChanged (PlayerEngineState.Playing);
+            bp_play (handle);
         }
         
         public override void Pause ()
         {
-            gst_playback_pause (handle);
-            OnStateChanged (PlayerEngineState.Paused);
+            bp_pause (handle);
+        }
+        
+        public override void VideoExpose (IntPtr window, bool direct)
+        {
+            bp_video_window_expose (handle, window, direct);
         }
 
         public override IntPtr [] GetBaseElements ()
         {
             IntPtr [] elements = new IntPtr[3];
             
-            if (gst_playback_get_pipeline_elements (handle, out elements[0], out elements[1], out elements[2])) {
+            if (bp_get_pipeline_elements (handle, out elements[0], out elements[1], out elements[2])) {
                 return elements;
             }
             
             return null;
         }
 
-        private void OnEos (IntPtr engine)
+        private void OnEos (IntPtr player)
         {
-            Close ();
-            OnEventChanged (PlayerEngineEvent.EndOfStream);
+            Close (false);
+            OnEventChanged (PlayerEvent.EndOfStream);
         }
         
-        private void OnIterate (IntPtr engine)
+        private void OnIterate (IntPtr player)
         {
-            OnEventChanged (PlayerEngineEvent.Iterate);
+            OnEventChanged (PlayerEvent.Iterate);
         }
         
-        private void OnError (IntPtr engine, uint domain, int code, IntPtr error, IntPtr debug)
+        private void OnStateChange (IntPtr player, GstState old_state, GstState new_state, GstState pending_state)
         {
-            Close ();
+            if (old_state == GstState.Ready && new_state == GstState.Paused && pending_state == GstState.Playing) {
+                OnStateChanged (PlayerState.Loaded);
+                return;
+            } else if (old_state == GstState.Paused && new_state == GstState.Playing && pending_state == GstState.VoidPending) {
+                if (CurrentState == PlayerState.Loaded) {
+                    OnEventChanged (PlayerEvent.StartOfStream);
+                }
+                OnStateChanged (PlayerState.Playing);
+                return;
+            } else if (CurrentState == PlayerState.Playing && old_state == GstState.Playing && new_state == GstState.Paused) {
+                OnStateChanged (PlayerState.Paused);
+                return;
+            }
+        }
+        
+        private void OnError (IntPtr player, uint domain, int code, IntPtr error, IntPtr debug)
+        {
+            Close (true);
             
             string error_message = error == IntPtr.Zero
                 ? Catalog.GetString ("Unknown Error")
@@ -232,22 +270,22 @@ namespace Banshee.GStreamer
                 Log.Error (String.Format("GStreamer library error: {0}", (GstLibraryError) code), false);
             }
             
-            OnEventChanged (PlayerEngineEvent.Error, error_message);
+            OnEventChanged (new PlayerEventErrorArgs (error_message));
         }
         
-        private void OnBuffering (IntPtr engine, int progress)
+        private void OnBuffering (IntPtr player, int progress)
         {
             if (buffering_finished && progress >= 100) {
                 return;
             }
             
             buffering_finished = progress >= 100;
-            OnEventChanged (PlayerEngineEvent.Buffering, Catalog.GetString ("Buffering"), (double) progress / 100.0);
+            OnEventChanged (new PlayerEventBufferingArgs ((double) progress / 100.0));
         }
         
-        private void OnTagFound (string tagName, ref GLib.Value value, IntPtr userData)
+        private void OnTagFound (IntPtr player, string tagName, ref GLib.Value value)
         {
-            OnTagFound(ProcessNativeTagResult (tagName, ref value));
+            OnTagFound (ProcessNativeTagResult (tagName, ref value));
         }
             
         private static StreamTag ProcessNativeTagResult (string tagName, ref GLib.Value valueRaw)
@@ -276,32 +314,32 @@ namespace Banshee.GStreamer
         }
         
         public override ushort Volume {
-            get { return (ushort)gst_playback_get_volume (handle); }
+            get { return (ushort)bp_get_volume (handle); }
             set { 
                 if ((IntPtr)handle == IntPtr.Zero) {
                     pending_volume = (short)value;
                     return;
                 }
                 
-                gst_playback_set_volume (handle, (int)value);
-                OnEventChanged (PlayerEngineEvent.Volume);
+                bp_set_volume (handle, (int)value);
+                OnEventChanged (PlayerEvent.Volume);
             }
         }
         
         public override uint Position {
-            get { return (uint)gst_playback_get_position(handle); }
+            get { return (uint)bp_get_position(handle); }
             set { 
-                gst_playback_set_position(handle, (ulong)value);
-                OnEventChanged (PlayerEngineEvent.Seek);
+                bp_set_position(handle, (ulong)value);
+                OnEventChanged (PlayerEvent.Seek);
             }
         }
         
         public override bool CanSeek {
-            get { return gst_playback_can_seek (handle); }
+            get { return bp_can_seek (handle); }
         }
         
         public override uint Length {
-            get { return (uint)gst_playback_get_duration (handle); }
+            get { return (uint)bp_get_duration (handle); }
         }
         
         public override string Id {
@@ -312,12 +350,37 @@ namespace Banshee.GStreamer
             get { return "GStreamer 0.10"; }
         }
         
+        private bool? supports_equalizer = null;
         public override bool SupportsEqualizer {
-            get { return gst_equalizer_is_supported (handle); }
+            get { 
+                if (supports_equalizer == null) {
+                    supports_equalizer = bp_equalizer_is_supported (handle); 
+                }
+                
+                return supports_equalizer.Value;
+            }
         }
-    
+        
+        private bool? supports_video = null;
+        public override bool SupportsVideo {
+            get { 
+                if (supports_video == null) {
+                    supports_video = bp_video_is_supported (handle); 
+                }
+                
+                return supports_video.Value;
+            }
+        }
+        
+        public override IntPtr VideoWindow {
+            set { bp_video_set_window (handle, value); }
+        }
+        
         public double AmplifierLevel {
-            set { gst_equalizer_set_preamp_level (handle, value); }
+            set {
+                double db = Math.Pow (10.0, value / 20.0);
+                bp_equalizer_set_preamp_level (handle, db);
+            }
         }
         
         public int [] BandRange {
@@ -325,7 +388,7 @@ namespace Banshee.GStreamer
                 int min = -1;
                 int max = -1;
                 
-                gst_equalizer_get_bandrange (handle, out min, out max);
+                bp_equalizer_get_bandrange (handle, out min, out max);
                 
                 return new int [] { min, max };
             }
@@ -334,7 +397,7 @@ namespace Banshee.GStreamer
         public uint [] EqualizerFrequencies {
             get {
                 double [] freq = new double[10];
-                gst_equalizer_get_frequencies (handle, out freq);
+                bp_equalizer_get_frequencies (handle, out freq);
                 
                 uint [] ret = new uint[freq.Length];
                 for (int i = 0; i < freq.Length; i++) {
@@ -347,7 +410,7 @@ namespace Banshee.GStreamer
         
         public void SetEqualizerGain (uint band, double gain)
         {
-            gst_equalizer_set_gain (handle, band, gain);
+            bp_equalizer_set_gain (handle, band, gain);
         }
         
         private static string [] source_capabilities = { "file", "http", "cdda" };
@@ -361,88 +424,97 @@ namespace Banshee.GStreamer
         }
         
         [DllImport ("libbanshee")]
-        private static extern IntPtr gst_playback_new ();
+        private static extern IntPtr bp_new ();
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_free (HandleRef engine);
+        private static extern void bp_destroy (HandleRef player);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_set_eos_callback (HandleRef engine, GstPlaybackEosCallback cb);
+        private static extern void bp_set_eos_callback (HandleRef player, BansheePlayerEosCallback cb);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_set_error_callback (HandleRef engine, GstPlaybackErrorCallback cb);
+        private static extern void bp_set_error_callback (HandleRef player, BansheePlayerErrorCallback cb);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_set_state_changed_callback (HandleRef engine, 
-            GstPlaybackStateChangedCallback cb);
+        private static extern void bp_set_state_changed_callback (HandleRef player, 
+            BansheePlayerStateChangedCallback cb);
             
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_set_iterate_callback (HandleRef engine,
-            GstPlaybackIterateCallback cb);
+        private static extern void bp_set_iterate_callback (HandleRef player,
+            BansheePlayerIterateCallback cb);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_set_buffering_callback (HandleRef engine,
-            GstPlaybackBufferingCallback cb);
+        private static extern void bp_set_buffering_callback (HandleRef player,
+            BansheePlayerBufferingCallback cb);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_set_tag_found_callback (HandleRef engine,
+        private static extern void bp_set_tag_found_callback (HandleRef player,
             GstTaggerTagFoundCallback cb);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_open (HandleRef engine, IntPtr uri);
+        private static extern bool bp_open (HandleRef player, IntPtr uri);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_stop (HandleRef engine);
+        private static extern void bp_stop (HandleRef player, bool nullstate);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_pause (HandleRef engine);
+        private static extern void bp_pause (HandleRef player);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_play (HandleRef engine);
+        private static extern void bp_play (HandleRef player);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_set_volume (HandleRef engine, int volume);
+        private static extern void bp_set_volume (HandleRef player, int volume);
         
         [DllImport("libbanshee")]
-        private static extern int gst_playback_get_volume (HandleRef engine);
+        private static extern int bp_get_volume (HandleRef player);
         
         [DllImport ("libbanshee")]
-        private static extern bool gst_playback_can_seek (HandleRef engine);
+        private static extern bool bp_can_seek (HandleRef player);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_set_position (HandleRef engine, ulong time_ms);
+        private static extern bool bp_set_position (HandleRef player, ulong time_ms);
         
         [DllImport ("libbanshee")]
-        private static extern ulong gst_playback_get_position (HandleRef engine);
+        private static extern ulong bp_get_position (HandleRef player);
         
         [DllImport ("libbanshee")]
-        private static extern ulong gst_playback_get_duration (HandleRef engine);
+        private static extern ulong bp_get_duration (HandleRef player);
         
         [DllImport ("libbanshee")]
-        private static extern bool gst_playback_get_pipeline_elements (HandleRef engine, out IntPtr playbin,
+        private static extern bool bp_get_pipeline_elements (HandleRef player, out IntPtr playbin,
             out IntPtr audiobin, out IntPtr audiotee);
             
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_set_gdk_window (HandleRef engine, IntPtr window);
+        private static extern void bp_set_application_gdk_window (HandleRef player, IntPtr window);
+        
+        [DllImport ("libbanshee")]
+        private static extern bool bp_video_is_supported (HandleRef player);
+        
+        [DllImport ("libbanshee")]
+        private static extern void bp_video_set_window (HandleRef player, IntPtr window);
+        
+        [DllImport ("libbanshee")]
+        private static extern void bp_video_window_expose (HandleRef player, IntPtr window, bool direct);
                                                                    
         [DllImport ("libbanshee")]
-        private static extern void gst_playback_get_error_quarks (out uint core, out uint library, 
+        private static extern void bp_get_error_quarks (out uint core, out uint library, 
             out uint resource, out uint stream);
         
         [DllImport ("libbanshee")]
-        private static extern bool gst_equalizer_is_supported (HandleRef engine);
+        private static extern bool bp_equalizer_is_supported (HandleRef player);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_equalizer_set_preamp_level (HandleRef engine, double level);
+        private static extern void bp_equalizer_set_preamp_level (HandleRef player, double level);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_equalizer_set_gain (HandleRef engine, uint bandnum, double gain);
+        private static extern void bp_equalizer_set_gain (HandleRef player, uint bandnum, double gain);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_equalizer_get_bandrange (HandleRef engine, out int min, out int max);
+        private static extern void bp_equalizer_get_bandrange (HandleRef player, out int min, out int max);
         
         [DllImport ("libbanshee")]
-        private static extern void gst_equalizer_get_frequencies (HandleRef engine,
+        private static extern void bp_equalizer_get_frequencies (HandleRef player,
             [MarshalAs (UnmanagedType.LPArray)] out double [] freq);
     }
 }
