@@ -35,6 +35,7 @@ using Gtk;
 using Hyena.Gui;
 using Hyena.Data;
 using Hyena.Data.Gui;
+using Hyena.Widgets;
 
 using Banshee.ServiceStack;
 using Banshee.Sources;
@@ -53,7 +54,7 @@ using Banshee.Sources.Gui;
 
 namespace Nereid
 {
-    public class PlayerInterface : BaseClientWindow, IService, IDisposable, IHasTrackSelection, IHasSourceView
+    public class PlayerInterface : BaseClientWindow, IService, IDisposable, IHasSourceView
     {
         // Major Layout Components
         private VBox primary_vbox;
@@ -76,9 +77,7 @@ namespace Nereid
         {
             BuildPrimaryLayout ();
             ConnectEvents ();
-            LoadSettings ();
-            
-            ActionService.TrackActions.TrackSelector = this;
+
             ActionService.SourceActions.SourceView = this;
             
             composite_view.TrackView.HasFocus = true;
@@ -92,7 +91,7 @@ namespace Nereid
             if (track != null) {
                 // Translators: this is the window title when a track is playing
                 //              {0} is the track title, {1} is the artist name
-                Title = String.Format (Catalog.GetString ("{0} ({1})"), 
+                Title = String.Format (Catalog.GetString ("{0} by {1}"), 
                     track.DisplayTrackTitle, track.DisplayArtistName);
             } else {
                 Title = Catalog.GetString ("Banshee Media Player");
@@ -120,7 +119,7 @@ namespace Nereid
         {
             primary_vbox = new VBox ();
             
-            Widget menu = ActionService.UIManager.GetWidget ("/MainMenu");
+            Widget menu = new MainMenu ();
             menu.Show ();
             primary_vbox.PackStart (menu, false, false, 0);
            
@@ -147,6 +146,10 @@ namespace Nereid
             
             primary_vbox.PackStart (toolbar_alignment, false, false, 0);
             
+            Widget next_button = new NextButton (ActionService);
+            next_button.Show ();
+            ActionService.PopulateToolbarPlaceholder (header_toolbar, "/HeaderToolbar/NextArrowButton", next_button);
+            
             ConnectedSeekSlider seek_slider = new ConnectedSeekSlider ();
             seek_slider.Show ();
             ActionService.PopulateToolbarPlaceholder (header_toolbar, "/HeaderToolbar/SeekSlider", seek_slider);
@@ -165,6 +168,7 @@ namespace Nereid
             VBox source_box = new VBox ();
             
             views_pane = new HPaned ();
+            PersistentPaneController.Control (views_pane, SourceViewWidth);
             view_container = new ViewContainer ();
             
             source_view = new SourceView ();
@@ -182,7 +186,7 @@ namespace Nereid
             source_view.SetSizeRequest (125, -1);
             view_container.SetSizeRequest (425, -1);
             
-            views_pane.Pack1 (source_box, true, false);
+            views_pane.Pack1 (source_box, false, false);
             views_pane.Pack2 (view_container, true, false);
             
             source_box.ShowAll ();
@@ -198,10 +202,19 @@ namespace Nereid
             footer_toolbar.ShowArrow = false;
             footer_toolbar.ToolbarStyle = ToolbarStyle.BothHoriz;
 
+            EventBox status_event_box = new EventBox ();
+            status_event_box.ButtonPressEvent += delegate (object o, ButtonPressEventArgs args) {
+                Source source = ServiceManager.SourceManager.ActiveSource;
+                if (source != null) {
+                    source.CycleStatusFormat ();
+                    UpdateSourceInformation ();
+                }
+            };
             status_label = new Label ();
+            status_event_box.Add (status_label);
             
             Alignment status_align = new Alignment (0.5f, 0.5f, 1.0f, 1.0f);
-            status_align.Add (status_label);
+            status_align.Add (status_event_box);
 
             RepeatActionButton repeat_button = new RepeatActionButton ();
             repeat_button.SizeAllocated += delegate (object o, Gtk.SizeAllocatedArgs args) {
@@ -216,15 +229,6 @@ namespace Nereid
         }
 
 #endregion
-
-#region Configuration Loading/Saving
-        
-        private void LoadSettings ()
-        {
-            views_pane.Position = SourceViewWidth.Get ();
-        }
-        
-#endregion
         
 #region Events and Logic Setup
         
@@ -233,7 +237,10 @@ namespace Nereid
             // Service events
             ServiceManager.SourceManager.ActiveSourceChanged += OnActiveSourceChanged;
             ServiceManager.SourceManager.SourceUpdated += OnSourceUpdated;
-            ServiceManager.PlayerEngine.EventChanged += OnPlayerEngineEventChanged;
+            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, 
+                PlayerEvent.StartOfStream |
+                PlayerEvent.TrackInfoUpdated |
+                PlayerEvent.EndOfStream);
             
             ActionService.TrackActions ["SearchForSameArtistAction"].Activated += OnProgrammaticSearch;
             ActionService.TrackActions ["SearchForSameAlbumAction"].Activated += OnProgrammaticSearch;
@@ -244,15 +251,16 @@ namespace Nereid
                 SourceViewWidth.Set (views_pane.Position);
             };
             
-            composite_view.TrackView.RowActivated += delegate (object o, RowActivatedArgs<TrackInfo> args) {
-                SetPlaybackControllerSource (ServiceManager.SourceManager.ActiveSource);
-                ServiceManager.PlayerEngine.OpenPlay (args.RowValue);
-            };
-
             source_view.RowActivated += delegate {
-                SetPlaybackControllerSource (ServiceManager.SourceManager.ActiveSource);
-                if (GtkUtilities.NoImportantModifiersAreSet (Gdk.ModifierType.ControlMask)) {
-                    ServiceManager.PlaybackController.First ();
+                Source source = ServiceManager.SourceManager.ActiveSource;
+                if (source is ITrackModelSource) {
+                    ServiceManager.PlaybackController.NextSource = (ITrackModelSource)source;
+                    // Allow changing the play source without stopping the current song by
+                    // holding ctrl when activating a source. After the song is done, playback will
+                    // continue from the new source.
+                    if (GtkUtilities.NoImportantModifiersAreSet (Gdk.ModifierType.ControlMask)) {
+                        ServiceManager.PlaybackController.Next ();
+                    }
                 }
             };
             
@@ -272,6 +280,7 @@ namespace Nereid
             view_container.SearchEntry.Ready = true;
         }
         
+        private TrackListModel previous_track_model = null;
         private void OnActiveSourceChanged (SourceEventArgs args)
         {
             Source source = ServiceManager.SourceManager.ActiveSource;
@@ -295,23 +304,24 @@ namespace Nereid
                 view_container.Content.ResetSource ();
             }
 
+            if (previous_track_model != null) {
+                previous_track_model.Reloaded -= HandleTrackModelReloaded;
+                previous_track_model = null;
+            }
+
+            if (source is ITrackModelSource) {
+                previous_track_model = (source as ITrackModelSource).TrackModel;
+                previous_track_model.Reloaded += HandleTrackModelReloaded;
+            }
+
             // Connect the source models to the views if possible
             if (source.Properties.Contains ("Nereid.SourceContents")) {
                 view_container.Content = source.Properties.Get<ISourceContents> ("Nereid.SourceContents");
                 view_container.Content.SetSource (source);
                 view_container.Show ();
             } else if (source is ITrackModelSource) {
-                if (composite_view.TrackModel != null) {
-                    composite_view.TrackModel.Reloaded -= HandleTrackModelReloaded;
-                }
-                composite_view.SetSource (source);
-                composite_view.TrackModel.Reloaded += HandleTrackModelReloaded;
-                PersistentColumnController column_controller = 
-                    composite_view.TrackView.ColumnController as PersistentColumnController;
-                if (column_controller != null) {
-                    column_controller.Source = source;
-                }
                 view_container.Content = composite_view;
+                view_container.Content.SetSource (source);
                 view_container.Show ();
             } else if (source is Hyena.Data.IObjectListModel) {
                 if (object_view == null) {
@@ -324,30 +334,35 @@ namespace Nereid
             } else {
                 view_container.Hide ();
             }
-            
+
+            // Associate the view with the model
+            if (view_container.Visible && view_container.Content is ITrackModelSourceContents) {
+                ITrackModelSourceContents track_content = view_container.Content as ITrackModelSourceContents;
+                source.Properties.Set<IListView<TrackInfo>>  ("Track.IListView", track_content.TrackView);
+                source.Properties.Set<IListView<ArtistInfo>> ("Artist.IListView", track_content.ArtistView);
+                source.Properties.Set<IListView<AlbumInfo>>  ("Album.IListView", track_content.AlbumView);
+            }
+
             view_container.Header.Visible = source.Properties.Contains ("Nereid.SourceContents.HeaderVisible") ?
                 source.Properties.Get<bool> ("Nereid.SourceContents.HeaderVisible") : true;
+
+            view_container.DiskUsageVisible = (source is IDiskUsageReporter);
             
-            UpdateStatusBar ();
+            UpdateSourceInformation ();
             view_container.SearchEntry.Ready = true;
         }
 
         private void OnSourceUpdated (SourceEventArgs args)
         {
             if (args.Source == ServiceManager.SourceManager.ActiveSource) {
-                UpdateStatusBar ();
+                UpdateSourceInformation ();
+                view_container.Title = args.Source.Name;
             }
         }
         
-        private void OnPlayerEngineEventChanged (object o, PlayerEngineEventArgs args) 
+        private void OnPlayerEvent (PlayerEventArgs args) 
         {
-            switch (args.Event) {
-                case PlayerEngineEvent.StartOfStream:
-                case PlayerEngineEvent.TrackInfoUpdated:
-                case PlayerEngineEvent.EndOfStream:
-                    UpdateTitle ();
-                    break;
-            }
+            UpdateTitle ();
         }
         
 #endregion
@@ -385,7 +400,7 @@ namespace Nereid
 #region Implement Interfaces
 
         // IHasTrackSelection
-        public IEnumerable<TrackInfo> GetSelectedTracks ()
+        /*public IEnumerable<TrackInfo> GetSelectedTracks ()
         {
             return new ModelSelection<TrackInfo> (composite_view.TrackModel, composite_view.TrackView.Selection);
         }
@@ -394,9 +409,9 @@ namespace Nereid
             get { return composite_view.TrackView.SelectionProxy; }
         }
 
-        public TrackListDatabaseModel TrackModel {
-            get { return composite_view.TrackModel as TrackListDatabaseModel; }
-        }
+        public DatabaseTrackListModel TrackModel {
+            get { return composite_view.TrackModel as DatabaseTrackListModel; }
+        }*/
 
         // IHasSourceView
         public Source HighlightedSource {
@@ -463,28 +478,34 @@ namespace Nereid
 
         private void HandleTrackModelReloaded (object sender, EventArgs args)
         {
-            UpdateStatusBar ();
+            UpdateSourceInformation ();
         }
 
-        private void SetPlaybackControllerSource (Source source)
+        private void UpdateSourceInformation ()
         {
-            // Set the source from which to play to the current source since
-            // the user manually began playback from this source
-            if (!(source is ITrackModelSource)) {
-                source = ServiceManager.SourceManager.DefaultSource;
-            }
-            
-            ServiceManager.PlaybackController.Source = (ITrackModelSource)source;    
-        }
-
-        private void UpdateStatusBar ()
-        {
-            if (ServiceManager.SourceManager.ActiveSource == null) {
+            Source source = ServiceManager.SourceManager.ActiveSource;
+            if (source == null) {
                 status_label.Text = String.Empty;
                 return;
             }
 
-            status_label.Text = ServiceManager.SourceManager.ActiveSource.GetStatusText ();
+            if (source is IDiskUsageReporter) {
+                IDiskUsageReporter reporter = source as IDiskUsageReporter;
+
+                double fraction = (double)reporter.BytesUsed / (double)reporter.BytesCapacity;
+
+                view_container.DiskUsageBar.Fraction = fraction;
+                view_container.DiskUsageBar.Text = String.Format(
+                    Catalog.GetString("{0} of {1}"),
+                    new Hyena.Query.FileSizeQueryValue (reporter.BytesUsed).ToUserQuery (),
+                    new Hyena.Query.FileSizeQueryValue (reporter.BytesCapacity).ToUserQuery ()
+                );
+
+                //string tooltip = dapSource.DiskUsageString + " (" + dapSource.DiskAvailableString + ")";
+                //toolTips.SetTip (dapDiskUsageBar, tooltip, tooltip);
+            }
+
+            status_label.Text = source.GetStatusText ();
         }
 
 #endregion
