@@ -32,10 +32,12 @@ using System.IO;
 
 using Hyena;
 using Hyena.Data.Sqlite;
+using Hyena.Query;
 
 using Banshee.Base;
 using Banshee.Configuration.Schema;
 using Banshee.Database;
+using Banshee.Query;
 using Banshee.Sources;
 using Banshee.ServiceStack;
 using Banshee.Streaming;
@@ -80,17 +82,17 @@ namespace Banshee.Collection.Database
 
         public override void IncrementPlayCount ()
         {
-            if (Provider.Refresh (this)) {
+            if (ProviderRefresh ()) {
                 base.IncrementPlayCount ();
-                Save ();
+                Save (true, BansheeQuery.PlayCountField, BansheeQuery.LastPlayedField);
             }
         }
 
         public override void IncrementSkipCount ()
         {
-            if (Provider.Refresh (this)) {
+            if (ProviderRefresh ()) {
                 base.IncrementSkipCount ();
-                Save ();
+                Save (true, BansheeQuery.SkipCountField, BansheeQuery.LastSkippedField);
             }
         }
 
@@ -105,11 +107,6 @@ namespace Banshee.Collection.Database
                 db_track.CacheEntryId == CacheEntryId;
         }
         
-        public override void Save ()
-        {
-            Save (true);
-        }
-        
         public DatabaseArtistInfo Artist {
             get { return DatabaseArtistInfo.FindOrCreate (ArtistName); }
         }
@@ -118,7 +115,18 @@ namespace Banshee.Collection.Database
             get { return DatabaseAlbumInfo.FindOrCreate (Artist, AlbumTitle); }
         }
 
-        public void Save (bool notify)
+        private static bool notify_saved = true;
+        public static bool NotifySaved {
+            get { return notify_saved; }
+            set { notify_saved = value; }
+        }
+
+        public override void Save ()
+        {
+            Save (NotifySaved);
+        }
+
+        public void Save (bool notify, params QueryField [] fields_changed)
         {
             // If either the artist or album changed, 
             if (ArtistId == 0 || AlbumId == 0 || artist_changed == true || album_changed == true) {
@@ -136,17 +144,27 @@ namespace Banshee.Collection.Database
             bool is_new = (TrackId == 0);
             if (is_new) DateAdded = DateUpdated;
 
-            Provider.Save (this);
+            ProviderSave ();
 
             if (notify) {
                 if (is_new) {
                     PrimarySource.NotifyTracksAdded ();
                 } else {
-                    PrimarySource.NotifyTracksChanged ();
+                    PrimarySource.NotifyTracksChanged (fields_changed);
                 }
             }
         }
         
+        protected virtual void ProviderSave ()
+        {
+            Provider.Save (this);
+        }
+        
+        protected virtual bool ProviderRefresh ()
+        {
+            return Provider.Refresh (this);
+        }
+        
         private int track_id;
         [DatabaseColumn ("TrackID", Constraints = DatabaseColumnConstraints.PrimaryKey)]
         public int TrackId {
@@ -169,15 +187,15 @@ namespace Banshee.Collection.Database
             set { PrimarySourceId = value.DbId; }
         }
 
-        [DatabaseColumn ("ArtistID")]
         private int artist_id;
+        [DatabaseColumn ("ArtistID")]
         public int ArtistId {
             get { return artist_id; }
             set { artist_id = value; }
         }
-
-        [DatabaseColumn ("AlbumID")]
+        
         private int album_id;
+        [DatabaseColumn ("AlbumID")]
         public int AlbumId {
             get { return album_id; }
             set { album_id = value; }
@@ -187,7 +205,8 @@ namespace Banshee.Collection.Database
         public override string ArtistName {
             get { return base.ArtistName; }
             set {
-                if (value == ArtistName)
+                value = CleanseString (value, ArtistName);
+                if (value == null)
                     return;
 
                 base.ArtistName = value;
@@ -199,7 +218,8 @@ namespace Banshee.Collection.Database
         public override string AlbumTitle {
             get { return base.AlbumTitle; }
             set {
-                if (value == AlbumTitle)
+                value = CleanseString (value, AlbumTitle);
+                if (value == null)
                     return;
 
                 base.AlbumTitle = value;
@@ -207,6 +227,23 @@ namespace Banshee.Collection.Database
             }
         }
         
+        private static string CleanseString (string input, string old_val)
+        {
+            if (input == old_val)
+                return null;
+                    
+            if (input != null)
+                input = input.Trim ();
+                
+            if (input == String.Empty)
+                return null;
+                
+            if (input == old_val)
+                return null;
+            
+            return input;
+        }
+        
         private int tag_set_id;
         [DatabaseColumn]
         public int TagSetID {
@@ -351,10 +388,16 @@ namespace Banshee.Collection.Database
             set { base.Comment = value; }
         }
         
-        [DatabaseColumn]
+        [DatabaseColumn ("Rating")]
+        private int rating;
         public override int Rating {
-            get { return base.Rating; }
-            set { base.Rating = value; }
+            get { return rating; }
+            set {
+                if (rating != value) {
+                    rating = value;
+                    Save (true, BansheeQuery.RatingField);
+                }
+            }
         }
         
         [DatabaseColumn]
@@ -408,6 +451,8 @@ namespace Banshee.Collection.Database
             }
         }
 
+#region Implement ICacheableItem
+
         private long cache_entry_id;
         public long CacheEntryId {
             get { return cache_entry_id; }
@@ -420,6 +465,8 @@ namespace Banshee.Collection.Database
             set { cache_model_id = value; }
         }
 
+#endregion
+
         private void UpdateUri ()
         {
             if (Uri == null && uri_type_set && UriField != null && PrimarySource != null) {
@@ -459,6 +506,12 @@ namespace Banshee.Collection.Database
             "SELECT TrackID FROM CoreTracks WHERE PrimarySourceId IN (?) AND (Uri = ? OR Uri = ?) LIMIT 1"
         );
 
+        public static int GetTrackIdForUri (SafeUri uri, string relative_path, int [] primary_sources)
+        {
+            return ServiceManager.DbConnection.Query<int> (check_command,
+                primary_sources, relative_path, uri.AbsoluteUri);
+        }
+
         public static int GetTrackIdForUri (string relative_path, int [] primary_sources)
         {
             return ServiceManager.DbConnection.Query<int> (check_command,
