@@ -54,8 +54,8 @@ namespace Banshee.PlaybackController
         private TrackInfo changing_to_track;
         private bool raise_started_after_transition = false;
         private bool transition_track_started = false;
-        
-        //private Random random = new Random ();
+        private int consecutive_errors;
+        private uint error_transition_id;
     
         private PlaybackShuffleMode shuffle_mode;
         private PlaybackRepeatMode repeat_mode;
@@ -63,6 +63,7 @@ namespace Banshee.PlaybackController
         
         private PlayerEngineService player_engine;
         private ITrackModelSource source;
+        private ITrackModelSource next_source;
         
         private event PlaybackControllerStoppedHandler dbus_stopped;
         event PlaybackControllerStoppedHandler IPlaybackController.Stopped {
@@ -72,6 +73,7 @@ namespace Banshee.PlaybackController
         
         public event EventHandler Stopped;
         public event EventHandler SourceChanged;
+        public event EventHandler NextSourceChanged;
         public event EventHandler TrackStarted;
         public event EventHandler Transition;
         
@@ -81,7 +83,12 @@ namespace Banshee.PlaybackController
             
             player_engine = ServiceManager.PlayerEngine;
             player_engine.PlayWhenIdleRequest += OnPlayerEnginePlayWhenIdleRequest;
-            player_engine.EventChanged += OnPlayerEngineEventChanged;
+            player_engine.ConnectEvent (OnPlayerEvent, 
+                PlayerEvent.EndOfStream | 
+                PlayerEvent.StartOfStream |
+                PlayerEvent.StateChange |
+                PlayerEvent.Error, 
+                true);
         }
         
         protected virtual void InstantiateStacks ()
@@ -95,23 +102,31 @@ namespace Banshee.PlaybackController
             Next ();
         }
         
-        private void OnPlayerEngineEventChanged (object o, PlayerEngineEventArgs args)
+        private void OnPlayerEvent (PlayerEventArgs args)
         {
             switch (args.Event) {
-                case PlayerEngineEvent.EndOfStream:
-                    if (!StopWhenFinished) {
-                        if (RepeatMode == PlaybackRepeatMode.RepeatSingle) {
-                            QueuePlayTrack ();
-                        } else {
-                            Next ();
-                        }
-                    } else {
+                case PlayerEvent.StartOfStream:
+                    consecutive_errors = 0;
+                    break;
+                case PlayerEvent.EndOfStream:
+                    EosTransition ();
+                       break;
+                case PlayerEvent.Error:
+                    if (++consecutive_errors >= 20) {
+                        consecutive_errors = 0;
+                        player_engine.Close (false);
                         OnStopped ();
+                        break;
                     }
                     
-                    StopWhenFinished = false;
+                    CancelErrorTransition ();
+                    Application.RunTimeout (500, EosTransition);
                     break;
-                case PlayerEngineEvent.StartOfStream:
+                case PlayerEvent.StateChange:
+                    if (((PlayerEventStateChangeArgs)args).Current != PlayerState.Loading) {
+                        break;
+                    }
+                       
                     TrackInfo track = player_engine.CurrentTrack;
                     if (changing_to_track != track && track != null) {
                         CurrentTrack = track;
@@ -129,8 +144,36 @@ namespace Banshee.PlaybackController
             }       
         }
         
+        private void CancelErrorTransition ()
+        {
+            if (error_transition_id > 0) {
+                Application.IdleTimeoutRemove (error_transition_id);
+                error_transition_id = 0;
+            }
+        }
+        
+        private bool EosTransition ()
+        {
+            if (!StopWhenFinished) {
+                if (RepeatMode == PlaybackRepeatMode.RepeatSingle) {
+                    QueuePlayTrack ();
+                } else {
+                    Next ();
+                }
+            } else {
+                OnStopped ();
+            }
+            
+            StopWhenFinished = false;
+            return false;
+        }
+        
         public void First ()
         {
+            CancelErrorTransition ();
+            
+            Source = NextSource;
+            
             // This and OnTransition() below commented out b/c of BGO #524556
             //raise_started_after_transition = true;
             
@@ -150,6 +193,9 @@ namespace Banshee.PlaybackController
         
         public void Next (bool restart)
         {
+            CancelErrorTransition ();
+            
+            Source = NextSource;
             raise_started_after_transition = true;
 
             player_engine.IncrementLastPlayed ();
@@ -170,6 +216,9 @@ namespace Banshee.PlaybackController
         
         public void Previous (bool restart)
         {
+            CancelErrorTransition ();
+            
+            Source = NextSource;
             raise_started_after_transition = true;
 
             player_engine.IncrementLastPlayed ();
@@ -313,6 +362,14 @@ namespace Banshee.PlaybackController
             }
         }
         
+        protected void OnNextSourceChanged ()
+        {
+            EventHandler handler = NextSourceChanged;
+            if (handler != null) {
+                handler (this, EventArgs.Empty);
+            }
+        }
+        
         protected virtual void OnTrackStarted ()
         {
             EventHandler handler = TrackStarted;
@@ -337,6 +394,7 @@ namespace Banshee.PlaybackController
             
             set {
                 if (source != value) {
+                    NextSource = value;
                     source = value;
                     source_set_at = DateTime.Now;
                     OnSourceChanged ();
@@ -344,6 +402,20 @@ namespace Banshee.PlaybackController
             }
         }
         
+        public ITrackModelSource NextSource {
+            get { return next_source ?? Source; }
+            set {
+                if (next_source != value) {
+                    next_source = value;
+                    OnNextSourceChanged ();
+                    
+                    if (!player_engine.IsPlaying ()) {
+                        Source = next_source;
+                    }
+                }
+            }
+        }
+        
         public PlaybackShuffleMode ShuffleMode {
             get { return shuffle_mode; }
             set { shuffle_mode = value; }
