@@ -34,26 +34,27 @@ using Mono.Addins;
 
 using Hyena;
 using Banshee.Base;
-using Banshee.AudioProfiles;
+using Banshee.MediaProfiles;
 using Banshee.Sources;
-using Banshee.SmartPlaylist;
 using Banshee.Database;
 using Banshee.MediaEngine;
 using Banshee.PlaybackController;
 using Banshee.Library;
+using Banshee.Hardware;
 
 namespace Banshee.ServiceStack
 {
     public static class ServiceManager
     {
         private static Dictionary<string, IService> services = new Dictionary<string, IService> ();
+        private static Dictionary<string, IExtensionService> extension_services = new Dictionary<string, IExtensionService> ();
         private static Stack<IService> dispose_services = new Stack<IService> ();
         private static List<Type> service_types = new List<Type> ();
         private static ExtensionNodeList extension_nodes;
         
         private static bool is_initialized = false;
         private static readonly object self_mutex = new object ();
-        
+
         public static event EventHandler StartupBegin;
         public static event EventHandler StartupFinished;
         public static event ServiceStartedHandler ServiceStarted;
@@ -62,14 +63,16 @@ namespace Banshee.ServiceStack
         {
             RegisterService<DBusServiceManager> ();
             RegisterService<BansheeDbConnection> ();
+            RegisterService<Banshee.Preferences.PreferenceService> ();
             RegisterService<SourceManager> ();
-            RegisterService<SmartPlaylistCore> ();
-            RegisterService<ProfileManager> ();
+            RegisterService<MediaProfileManager> ();
             RegisterService<PlayerEngineService> ();
+            RegisterService<TranscoderService> ();
             RegisterService<PlaybackControllerService> ();
             RegisterService<ImportSourceManager> ();
             RegisterService<LibraryImportManager> ();
             RegisterService<UserJobManager> ();
+            RegisterService<Banshee.Hardware.HardwareManager> ();
             
             AddinManager.Initialize (ApplicationContext.CommandLine.Contains ("uninstalled") 
                 ? "." : Paths.ApplicationData);
@@ -84,8 +87,14 @@ namespace Banshee.ServiceStack
                 AddinManager.Registry.Update (monitor);
             }
             
-            Banshee.Configuration.ConfigurationClient.Initialize ();
             extension_nodes = AddinManager.GetExtensionNodes ("/Banshee/ServiceManager/Service");
+
+            Application.ClientStarted += OnClientStarted;
+        }
+
+        private static void OnClientStarted (Client client)
+        {
+            DelayedInitialize ();
         }
         
         public static void Run()
@@ -96,48 +105,44 @@ namespace Banshee.ServiceStack
                 uint cumulative_timer_id = Log.InformationTimerStart ();
                 
                 foreach (Type type in service_types) {
-                    uint timer_id = Log.DebugTimerStart (); 
-                    IService service = (IService)Activator.CreateInstance (type);
-                    RegisterService (service);
+                    IService service = null;
                     
-                    Log.DebugTimerPrint (timer_id, String.Format (
-                        "Core service started ({0}, {{0}})", service.ServiceName));
-                    
-                    OnServiceStarted (service);
-                    
-                    if (service is IDisposable) {
-                        dispose_services.Push (service);
-                    }
+                    try {
+                        uint timer_id = Log.DebugTimerStart ();
+                        service = (IService)Activator.CreateInstance (type);
+                        RegisterService (service);
+                        
+                        Log.DebugTimerPrint (timer_id, String.Format (
+                            "Core service started ({0}, {{0}})", service.ServiceName));
+                        
+                        OnServiceStarted (service);
+                        
+                        if (service is IDisposable) {
+                            dispose_services.Push (service);
+                        }
 
-                    if (service is IInitializeService) {
-                        ((IInitializeService)service).Initialize ();
+                        if (service is IInitializeService) {
+                            ((IInitializeService)service).Initialize ();
+                        }
+                    } catch (Exception e) {
+                        if (service is IRequiredService) {
+                            Log.ErrorFormat ("Error initializing required service {0}",
+                                    service == null ? type.ToString () : service.ServiceName, false);
+                            throw;
+                        }
+                        
+                        Log.Warning (String.Format ("Service `{0}' not started: {1}", type.FullName, 
+                            e.InnerException != null ? e.InnerException.Message : e.Message));
+                        Log.Exception (e.InnerException ?? e);
                     }
                 }
                 
                 foreach (TypeExtensionNode node in extension_nodes) {
-                    IExtensionService service = null;
-                    
-                    try {
-                        uint timer_id = Log.DebugTimerStart ();
-                        
-                        service = (IExtensionService)node.CreateInstance (typeof (IExtensionService));
-                        service.Initialize ();
-                        RegisterService (service);
-                    
-                        Log.DebugTimerPrint (timer_id, String.Format (
-                            "Extension service started ({0}, {{0}})", service.ServiceName));
-                    
-                        OnServiceStarted (service);
-                    
-                        if (service is IDisposable) {
-                            dispose_services.Push (service);
-                        }
-                    } catch (Exception e) {
-                        Log.Warning (String.Format ("Extension `{0}' not started: {1}", 
-                            service == null ? node.Path : service.GetType ().FullName, e.Message));
-                    }
+                    StartExtension (node);
                 }
                 
+                AddinManager.AddExtensionNodeHandler ("/Banshee/ServiceManager/Service", OnExtensionChanged);
+                
                 is_initialized = true;
                 
                 Log.InformationTimerPrint (cumulative_timer_id, "All services are started {0}");
@@ -146,6 +151,89 @@ namespace Banshee.ServiceStack
             }
         }
         
+        private static void StartExtension (TypeExtensionNode node)
+        {
+            if (extension_services.ContainsKey (node.Path)) {
+                return;
+            }
+        
+            IExtensionService service = null;
+                    
+            try {
+                uint timer_id = Log.DebugTimerStart ();
+                
+                service = (IExtensionService)node.CreateInstance (typeof (IExtensionService));
+                service.Initialize ();
+                RegisterService (service);
+
+                DelayedInitialize (service);
+            
+                Log.DebugTimerPrint (timer_id, String.Format (
+                    "Extension service started ({0}, {{0}})", service.ServiceName));
+            
+                OnServiceStarted (service);
+                
+                extension_services.Add (node.Path, service);
+            
+                if (service is IDisposable) {
+                    dispose_services.Push (service);
+                }
+            } catch (Exception e) {
+                Log.Exception (e.InnerException ?? e);
+                Log.Warning (String.Format ("Extension `{0}' not started: {1}", 
+                    service == null ? node.Path : service.GetType ().FullName, e.Message));
+            }
+        }
+        
+        private static void OnExtensionChanged (object o, ExtensionNodeEventArgs args) 
+        {
+            lock (self_mutex) {
+                TypeExtensionNode node = (TypeExtensionNode)args.ExtensionNode;
+                
+                if (args.Change == ExtensionChange.Add) {
+                    StartExtension (node);
+                } else if (args.Change == ExtensionChange.Remove && extension_services.ContainsKey (node.Path)) {
+                    IExtensionService service = extension_services[node.Path];
+                    extension_services.Remove (node.Path);
+                    services.Remove (service.ServiceName);
+                    ((IDisposable)service).Dispose ();
+                    
+                    Log.DebugFormat ("Extension service disposed ({0})", service.ServiceName);
+                    
+                    // Rebuild the dispose stack excluding the extension service
+                    IService [] tmp_services = new IService[dispose_services.Count - 1];
+                    int count = tmp_services.Length;
+                    foreach (IService tmp_service in dispose_services) {
+                        if (tmp_service != service) {
+                            tmp_services[--count] = tmp_service;
+                        }
+                    }
+                    dispose_services = new Stack<IService> (tmp_services);
+                }
+            }
+        }
+
+        private static bool delayed_initialized, have_client;
+        private static void DelayedInitialize ()
+        {
+            lock (self_mutex) {
+                if (!delayed_initialized) {
+                    have_client = true;
+                    foreach (IService service in services.Values) {
+                        DelayedInitialize (service);
+                    }
+                    delayed_initialized = true;
+                }
+            }
+        }
+        
+        private static void DelayedInitialize (IService service)
+        {
+            if (have_client && !delayed_initialized && service is IDelayedInitializeService) {
+                ((IDelayedInitializeService)service).DelayedInitialize ();
+            }
+        }
+        
         public static void Shutdown ()
         {
             lock (self_mutex) {
@@ -256,8 +344,8 @@ namespace Banshee.ServiceStack
             get { return (BansheeDbConnection)Get ("DbConnection"); }
         }
 
-        public static ProfileManager ProfileManager {
-            get { return (ProfileManager)Get ("ProfileManager"); }
+        public static MediaProfileManager MediaProfileManager {
+            get { return Get<MediaProfileManager> (); }
         }
         
         public static SourceManager SourceManager {
@@ -271,5 +359,9 @@ namespace Banshee.ServiceStack
         public static PlaybackControllerService PlaybackController {
             get { return (PlaybackControllerService)Get ("PlaybackController"); }
         }
+        
+        public static HardwareManager HardwareManager {
+            get { return Get<HardwareManager> (); }
+        }
     }
 }
