@@ -86,7 +86,7 @@ namespace Banshee.Sources
 
         protected DatabaseTrackListModel DatabaseTrackModel {
             get {
-                return track_model ?? track_model = new DatabaseTrackListModel (ServiceManager.DbConnection, TrackProvider);
+                return track_model ?? track_model = new DatabaseTrackListModel (ServiceManager.DbConnection, TrackProvider, this);
             }
             set { track_model = value; }
         }
@@ -109,8 +109,8 @@ namespace Banshee.Sources
             InitializeTrackModel ();
 
             if (HasArtistAlbum) {
-                artist_model = new DatabaseArtistListModel (DatabaseTrackModel, ServiceManager.DbConnection, UniqueId);
-                album_model = new DatabaseAlbumListModel (DatabaseTrackModel, artist_model, ServiceManager.DbConnection, UniqueId);
+                artist_model = new DatabaseArtistListModel (this, DatabaseTrackModel, ServiceManager.DbConnection, UniqueId);
+                album_model = new DatabaseAlbumListModel (this, DatabaseTrackModel, ServiceManager.DbConnection, UniqueId);
             }
 
             reload_limiter = new RateLimiter (RateLimitedReload);
@@ -141,7 +141,7 @@ namespace Banshee.Sources
         public override string FilterQuery {
             set {
                 base.FilterQuery = value;
-                DatabaseTrackModel.Filter = FilterQuery;
+                DatabaseTrackModel.UserQuery = FilterQuery;
                 ThreadAssist.SpawnFromMain (delegate {
                     Reload ();
                 });
@@ -172,12 +172,14 @@ namespace Banshee.Sources
             get { return DatabaseTrackModel; }
         }
         
-        public AlbumListModel AlbumModel {
-            get { return album_model; }
-        }
-        
-        public ArtistListModel ArtistModel {
-            get { return artist_model; }
+        public virtual IEnumerable<IFilterListModel> FilterModels {
+            get {
+                if (artist_model != null)
+                    yield return artist_model;
+                    
+                if (album_model != null)
+                    yield return album_model;
+            }
         }
         
         public virtual bool ShowBrowser { 
@@ -209,9 +211,9 @@ namespace Banshee.Sources
         {
             lock (track_model) {
                 DatabaseTrackModel.Reload ();
-                OnUpdated ();
-                Save ();
             }
+            OnUpdated ();
+            Save ();
         }
 
         public virtual bool HasDependencies {
@@ -328,11 +330,11 @@ namespace Banshee.Sources
             OnTracksChanged (null);
         }
 
-        protected virtual void OnTracksChanged (QueryField field)
+        protected virtual void OnTracksChanged (params QueryField [] fields)
         {
-            HandleTracksChanged (this, new TrackEventArgs (field));
+            HandleTracksChanged (this, new TrackEventArgs (fields));
             foreach (PrimarySource psource in PrimarySources) {
-                psource.NotifyTracksChanged (field);
+                psource.NotifyTracksChanged (fields);
             }
         }
 
@@ -397,7 +399,7 @@ namespace Banshee.Sources
 
         protected virtual void AfterInitialized ()
         {
-            DatabaseTrackModel.Initialize (TrackCache, artist_model, album_model);
+            DatabaseTrackModel.Initialize (TrackCache);
             OnSetupComplete ();
         }
 
@@ -462,6 +464,18 @@ namespace Banshee.Sources
             }
         }
 
+        protected void InvalidateCaches ()
+        {
+            track_model.InvalidateCache ();
+            
+            // TODO invalidate cache on all FilterModels
+            if (artist_model != null)
+                artist_model.InvalidateCache ();
+
+            if (album_model != null)
+                album_model.InvalidateCache ();
+        }
+
         protected virtual void PruneArtistsAlbums ()
         {
             //Console.WriteLine ("Pruning with {0}", PruneCommand.Text);
