@@ -47,10 +47,11 @@ using Banshee.Metadata;
 using Banshee.MediaEngine;
 using Banshee.Collection;
 using Banshee.ServiceStack;
+using Banshee.PlaybackController;
  
 namespace Banshee.Lastfm.Radio
 {
-    public class StationSource : Source, ITrackModelSource, IUnmapableSource, IDisposable
+    public class StationSource : Source, ITrackModelSource, IUnmapableSource, IDisposable, IBasicPlaybackController
     {
         private static string generic_name = Catalog.GetString ("Last.fm Station");
         
@@ -96,6 +97,10 @@ namespace Banshee.Lastfm.Radio
             get { return Convert.ToString (dbid); }
         }
         
+        public System.Collections.Generic.IEnumerable<Banshee.Collection.Database.IFilterListModel> FilterModels {
+            get { yield break; }
+        }
+        
         // For StationSources that already exist in the db
         protected StationSource (LastfmSource lastfm, int dbId, string name, string type, string arg, int playCount) : base (generic_name, name, 150)
         {
@@ -125,7 +130,7 @@ namespace Banshee.Lastfm.Radio
         {
             track_model = new MemoryTrackListModel ();
 
-            ServiceManager.PlayerEngine.StateChanged += OnPlayerStateChanged;
+            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, PlayerEvent.StateChange);
             lastfm.Connection.StateChanged += HandleConnectionStateChanged;
 
             Properties.SetString ("GtkActionPath", "/LastfmStationSourcePopup");
@@ -137,7 +142,7 @@ namespace Banshee.Lastfm.Radio
 
         public void Dispose ()
         {
-            ServiceManager.PlayerEngine.StateChanged -= OnPlayerStateChanged;
+            ServiceManager.PlayerEngine.DisconnectEvent (OnPlayerEvent);
             lastfm.Connection.StateChanged -= HandleConnectionStateChanged;
         }
 
@@ -232,13 +237,13 @@ namespace Banshee.Lastfm.Radio
             }
         }
 
-        protected override void SetStatus (string message, bool error)
+        public override void SetStatus (string message, bool error)
         {
             base.SetStatus (message, error);
             LastfmSource.SetStatus (status_message, lastfm, error, ConnectionState.Connected);
         }
 
-        private void SetStatus (string message, bool error, ConnectionState state)
+        public void SetStatus (string message, bool error, ConnectionState state)
         {
             base.SetStatus (message, error);
             LastfmSource.SetStatus (status_message, lastfm, error, state);
@@ -270,6 +275,30 @@ namespace Banshee.Lastfm.Radio
                     current_track = i;
             }
         }
+        
+#region IBasicPlaybackController
+
+        void IBasicPlaybackController.First ()
+        {
+            ((IBasicPlaybackController)this).Next (false);
+        }
+        
+        private bool playback_requested;    
+        void IBasicPlaybackController.Next (bool restart)
+        {
+            TrackInfo next = NextTrack;
+            if (next != null) {
+                ServiceManager.PlayerEngine.OpenPlay (next);
+            }  else {
+                playback_requested = true;
+            }
+        }
+        
+        void IBasicPlaybackController.Previous (bool restart)
+        {
+        }
+        
+#endregion
 
         public TrackInfo NextTrack {
             get { return GetTrack (current_track + 1); }
@@ -325,10 +354,12 @@ namespace Banshee.Lastfm.Radio
                             track_model.Reload ();
                             OnUpdated ();
 
-                            /*if (playback_requested) {
-                                StartPlayback ();
+                            if (playback_requested) {
+                                if (this == ServiceManager.PlaybackController.Source ) {
+                                    ((IBasicPlaybackController)this).Next (false);
+                                }
                                 playback_requested = false;
-                            }*/
+                            }
                         });
                     }
                 } else {
@@ -339,9 +370,10 @@ namespace Banshee.Lastfm.Radio
             });
         }
 
-        private void OnPlayerStateChanged (object o, PlayerEngineStateArgs args)
+        private void OnPlayerEvent (PlayerEventArgs args)
         {
-            if (args.State == PlayerEngineState.Loaded && track_model.Contains (ServiceManager.PlayerEngine.CurrentTrack)) {
+            if (((PlayerEventStateChangeArgs)args).Current == PlayerState.Loaded && 
+                track_model.Contains (ServiceManager.PlayerEngine.CurrentTrack)) {
                 CurrentTrack = ServiceManager.PlayerEngine.CurrentTrack;
 
                 lock (track_model) {
