@@ -4,7 +4,7 @@
 // Author:
 //   Aaron Bockover <abockover@novell.com>
 //
-// Copyright (C) 2007 Novell, Inc.
+// Copyright (C) 2007-2008 Novell, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -33,7 +33,11 @@ using System.Collections;
 using System.Collections.Generic;
 #endif
 
+#if NET_1_1
+namespace System.Collections
+#else
 namespace Hyena.Collections
+#endif
 {
 #if NET_1_1
     internal
@@ -87,6 +91,11 @@ namespace Hyena.Collections
                     : (Start + (End - Start)).CompareTo (
                         x.Start + (x.End - x.Start));
             }
+            
+            public override string ToString ()
+            {
+                return String.Format ("{0}-{1} ({2})", Start, End, Count);
+            }
 
             public int Count {
                 get { return End - Start + 1; }
@@ -135,7 +144,7 @@ namespace Hyena.Collections
 #else
             Range [] new_ranges = new Range[new_capacity];
             Array.Copy (ranges, 0, new_ranges, 0, ranges.Length);
-            new_ranges = ranges;
+            ranges = new_ranges;
 #endif
         }
         
@@ -243,7 +252,7 @@ namespace Hyena.Collections
             return min;
         }
         
-        private int FindRangeIndexForValue (int value)
+        public int FindRangeIndexForValue (int value)
         {
             return Array.BinarySearch (ranges, 0, range_count, new Range (value, -1));
         }
@@ -300,19 +309,16 @@ namespace Hyena.Collections
             
             return -1;
         }
-        
+
         public int this[int index] {
             get {
-                int offset = 0;
-                foreach (Range range in ranges) {
-                    if (index >= range.Start && index <= range.End) {
-                        return offset + (range.End - index);
+                for (int i = 0, cuml_count = 0; i < range_count && index >= 0; i++) {
+                    if (index < (cuml_count += ranges[i].Count)) {
+                        return ranges[i].End - (cuml_count - index) + 1;
                     }
-                    
-                    offset += range.End - range.Start;
                 }
                 
-                return -1;
+                throw new IndexOutOfRangeException (index.ToString ());
             }
         }
 
@@ -320,13 +326,27 @@ namespace Hyena.Collections
 
 #region ICollection Implementation
 
-        public void Add (int value)
+        public bool Add (int value)
         {
             if (!Contains (value)) {
                 generation++;
                 InsertRange (new Range (value, value));
                 index_count++;
+                return true;
             }
+            
+            return false;
+        }
+        
+        void
+#if NET_2_0
+        ICollection<int>.
+#else
+        ICollection.
+#endif
+        Add (int value)
+        {
+            Add (value);
         }
                 
         public bool Remove (int value)
@@ -414,4 +434,3 @@ namespace Hyena.Collections
 
     }
 }
-
